@@ -1,4 +1,4 @@
-__version__ = "0.4"
+__version__ = "0.4.1"
 
 from . import backtester
 from . import strategy
@@ -6,10 +6,4 @@ from . import fees
 from . import price_providers
 from . import metrics
 
-__all__ = [
-    "backtester",
-    "strategy",
-    "fees",
-    "price_providers",
-    "metrics"
-]
\ No newline at end of file
+__all__ = ["backtester", "strategy", "fees", "price_providers", "metrics"]
